@@ -6,6 +6,7 @@
 #include "config.h"
 #endif
 
+#include <array>
 #include <iostream>
 
 #include <dune/grid/yaspgrid.hh>
@@ -26,7 +27,8 @@ int main(int argc, char **argv)
     // make grid
     const int dim = 2;
     Dune::FieldVector<double,dim> L(1.0);
-    Dune::array<int,dim> N(Dune::fill_array<int,dim>(4));
+    std::array<int,dim> N;
+    std::fill(N.begin(), N.end(), 4);
     std::bitset<dim> periodic (false);
     periodic[0] = true;
     int overlap = 1;
