@@ -6,16 +6,17 @@
 
 #include <unistd.h>
 #include <iostream>
+#include <memory>
 #include <vector>
 
-#include <dune/grid/uggrid.hh>
-#include <dune/grid/common/mcmgmapper.hh>
-#include <dune/grid/io/file/vtk/vtkwriter.hh>
 #include <dune/common/parallel/mpihelper.hh>
 #include <dune/common/float_cmp.hh>
-#include <dune/grid/common/gridenums.hh>
-#include <dune/grid/utility/structuredgridfactory.hh>
 #include <dune/geometry/referenceelements.hh>
+#include <dune/grid/common/gridenums.hh>
+#include <dune/grid/common/mcmgmapper.hh>
+#include <dune/grid/io/file/vtk/vtkwriter.hh>
+#include <dune/grid/uggrid.hh>
+#include <dune/grid/utility/structuredgridfactory.hh>
 
 using namespace Dune;
 
@@ -61,7 +62,7 @@ public:
   }
 
   //! returns true if size per entity of given dim and codim is a constant
-  bool fixedsize (int dim, int codim) const
+  bool fixedSize (int dim, int codim) const
   {
     return true;
   }
@@ -123,35 +124,27 @@ private:
 template <class GridView>
 void checkIntersections(const GridView &gv)
 {
-  typedef typename GridView::template Codim<0>::Iterator Iterator;
-  typedef typename GridView::IntersectionIterator IntersectionIterator;
-
-  Iterator it = gv.template begin<0>();
-  const Iterator &endIt = gv.template end<0>();
-  // check the intersections
-  for (; it != endIt; ++it) {
-    if (it->partitionType() == Dune::GhostEntity)
+  for (const auto& element : elements(gv)) {
+    if (element.partitionType() == Dune::GhostEntity)
       continue;
 
-    IntersectionIterator isIt           = gv.ibegin(*it);
-    const IntersectionIterator &isEndIt = gv.iend(*it);
     unsigned int n = 0;
-    for (; isIt != isEndIt; ++isIt) {
-      isIt->boundary();
-      isIt->inside();
-      if (isIt->neighbor()) {
-        isIt->outside();
+    for (const auto& intersection : intersections(gv, element)) {
+      intersection.boundary();
+      intersection.inside();
+      if (intersection.neighbor()) {
+        intersection.outside();
       }
 
       ++ n;
     }
 
-    if (n != it->subEntities(1))
+    if (n != element.subEntities(1))
     {
 
       DUNE_THROW(Dune::InvalidStateException,
                  "Number of faces for non-ghost cell incorrect. Is "
-                 << n << " but should be " << it->subEntities(1));
+                 << n << " but should be " << element.subEntities(1));
     }
   }
 }
@@ -160,15 +153,10 @@ void checkIntersections(const GridView &gv)
 template <int codim, class GridView>
 void checkMappers(const GridView &gridView)
 {
-  typename GridView::template Codim<codim>::Iterator
-  it = gridView.template begin<codim>();
-  const typename GridView::template Codim<codim>::Iterator
-  &endIt = gridView.template end<codim>();
-
   // make sure the number of entities is the same for
   // gridView.size() and iterating through the grid
   int numEntities = 0;
-  for (; it != endIt; ++it)
+  for (DUNE_UNUSED const auto& entity : entities(gridView, Dune::Codim<codim>()))
     ++ numEntities;
   if (numEntities != gridView.size(codim)) {
     DUNE_THROW(InvalidStateException,
@@ -184,9 +172,8 @@ void checkMappers(const GridView &gridView)
 
   // make sure no entity has two indices
   std::vector<int> indices(numEntities, -100);
-  it = gridView.template begin<codim>();
-  for (; it != endIt; ++it) {
-    int i = mapper.index(*it);
+  for (const auto& entity : entities(gridView, Dune::Codim<codim>())) {
+    int i = mapper.index(entity);
     if (i < 0 || i >= numEntities) {
       DUNE_THROW(InvalidStateException,
                  gridView.comm().rank() + 1
@@ -258,13 +245,9 @@ void testCommunication(const GridView &gridView, bool isLeaf, bool printVTK=fals
 
   // write the partition type of each entity into the corrosponding
   // result array
-  typename GridView::template Codim<commCodim>::Iterator
-  it = gridView.template begin<commCodim>();
-  const typename GridView::template Codim<commCodim>::Iterator
-  &endIt = gridView.template end<commCodim>();
-  for (; it != endIt; ++it) {
-    entityIndex[mapper.index(*it)]   = mapper.index(*it);
-    partitionType[mapper.index(*it)] = it->partitionType();
+  for (const auto& entity : entities(gridView, Dune::Codim<commCodim>())) {
+    entityIndex[mapper.index(entity)]   = mapper.index(entity);
+    partitionType[mapper.index(entity)] = entity.partitionType();
   }
 
   // initialize data handle (marks the nodes where some data was
@@ -340,33 +323,26 @@ public:
 
     // write the partition type of each entity into the corresponding
     // result array
-    typename GridView::template Codim<0>::Iterator
-    it = gridView.template begin<0>();
-    const typename GridView::template Codim<0>::Iterator
-    &endIt = gridView.template end<0>();
-    for (; it != endIt; ++it) {
-      int numberOfSubEntities = it->subEntities(commCodim);
+    for (const auto& element : elements(gridView)) {
+      int numberOfSubEntities = element.subEntities(commCodim);
       for (int k = 0; k < numberOfSubEntities; k++)
       {
         typedef typename GridView::template Codim<0>::Entity Element;
-        typedef typename Element::template Codim<commCodim>::EntityPointer EntityPointer;
-        const EntityPointer entityPointer(it->template subEntity<commCodim>(k));
-        entityIndex[mapper.index(*entityPointer)]   = mapper.index(*entityPointer);
-        partitionType[mapper.index(*entityPointer)] = entityPointer->partitionType();
+        typedef typename Element::template Codim<commCodim>::Entity Entity;
+        const Entity entity(element.template subEntity<commCodim>(k));
+        entityIndex[mapper.index(entity)]   = mapper.index(entity);
+        partitionType[mapper.index(entity)] = entity.partitionType();
 
-        if (entityPointer->partitionType() == Dune::BorderEntity)
+        if (entity.partitionType() == Dune::BorderEntity)
         {
-          typedef typename GridView::template Codim<0>::Entity Element;
-          const typename Element::Geometry& geometry = it->geometry();
-          Dune::GeometryType gt = geometry.type();
+          const auto geometry = element.geometry();
+          const auto gt = geometry.type();
 
-          const typename Dune::ReferenceElementContainer<double, dim>::value_type&
-          referenceElement = Dune::ReferenceElements<double, dim>::general(gt);
-          const Dune::FieldVector<double, dim>&
-          entityGlobal = geometry.global(referenceElement.position(k, commCodim));
+          const auto& referenceElement = Dune::ReferenceElements<double, dim>::general(gt);
+          const auto entityGlobal = geometry.global(referenceElement.position(k, commCodim));
           std::cout << gridView.comm().rank()+1 << ": border codim "
                     << commCodim << " entity "
-                    << mapper.index(*entityPointer) << " (" << entityGlobal
+                    << mapper.index(entity) << " (" << entityGlobal
                     << ")" << std::endl;
         }
       }
@@ -403,7 +379,7 @@ class LoadBalance
     bool contains (int dim, int codim) const
     { return (codim == commCodim); }
 
-    bool fixedsize (int dim, int codim) const
+    bool fixedSize (int dim, int codim) const
     { return true; }
 
     template<class Entity>
@@ -451,17 +427,13 @@ public:
     std::vector<Position> dataVector(grid.leafGridView().size(commCodim));
 
     // fill the data vector
-    typedef typename Grid::LeafGridView LeafGV;
-    typedef typename LeafGV::template Codim<commCodim>::template Partition
-    <Dune::InteriorBorder_Partition>::Iterator Iterator;
-    const LeafGV& gv = grid.leafGridView();
-    Iterator it = gv.template begin<commCodim, Dune::InteriorBorder_Partition>();
-    const Iterator& endIt = gv.template end<commCodim, Dune::InteriorBorder_Partition>();
-    for (; it != endIt; ++it) {
-      int index = gv.indexSet().index(*it);
+    const auto& gv = grid.leafGridView();
+    for (const auto& entity : entities(gv, Dune::Codim<commCodim>(),
+                                       Dune::Partitions::interiorBorder)) {
+      int index = gv.indexSet().index(entity);
 
       // assign the position of the entity to the entry in the vector
-      dataVector[index] = it->geometry().center();
+      dataVector[index] = entity.geometry().center();
     }
 
     // balance the grid and the data
@@ -469,11 +441,11 @@ public:
     grid.loadBalance(dataHandle);
 
     // check for correctness
-    it = gv.template begin<commCodim, Dune::InteriorBorder_Partition>();
-    for (; it != endIt; ++it) {
-      int index = gv.indexSet().index(*it);
+    for (const auto& entity : entities(gv, Dune::Codim<commCodim>(),
+                                       Dune::Partitions::interiorBorder)) {
+      int index = gv.indexSet().index(entity);
 
-      const Position& position = it->geometry().center();
+      const auto position = entity.geometry().center();
 
       // compare the position with the balanced data
       for (int k = 0; k < dim; k++)
@@ -494,7 +466,7 @@ public:
 };
 
 template <int dim>
-void testParallelUG(bool localRefinement)
+void testParallelUG(bool simplexGrid, bool localRefinement, int refinementDim, bool refineUpperPart)
 {
   std::cout << "Testing parallel UGGrid for " << dim << "D\n";
 
@@ -508,21 +480,19 @@ void testParallelUG(bool localRefinement)
 
   Dune::FieldVector<double,dim> lowerLeft(0);
   Dune::FieldVector<double,dim> upperRight(1);
-  Dune::array<unsigned int, dim> elements;
-  std::fill(elements.begin(), elements.end(), 4);
-  shared_ptr<GridType> grid = structuredGridFactory.createCubeGrid(lowerLeft, upperRight, elements);
+  std::array<unsigned int, dim> numElements;
+  std::fill(numElements.begin(), numElements.end(), 4);
+  std::shared_ptr<GridType> grid;
+  if (simplexGrid)
+    grid = structuredGridFactory.createSimplexGrid(lowerLeft, upperRight, numElements);
+  else
+    grid = structuredGridFactory.createCubeGrid(lowerLeft, upperRight, numElements);
 
   //////////////////////////////////////////////////////
   // Distribute the grid
   //////////////////////////////////////////////////////
   LoadBalance::test(*grid);
 
-  std::cout << "Process " << grid->comm().rank() + 1
-            << " has " << grid->size(0)
-            << " elements and " << grid->size(dim)
-            << " nodes.\n";
-
-
   std::cout << "Process " << grid->comm().rank() + 1
             << " has " << grid->size(0)
             << " elements and " << grid->size(dim)
@@ -565,18 +535,18 @@ void testParallelUG(bool localRefinement)
   checkMappersWrapper<dim, 3, LeafGV>::check(leafGridView);
 
   // Test element and node communication on level view
-  testCommunication<typename GridType::LevelGridView, 0>(level0GridView, false);
-  testCommunication<typename GridType::LevelGridView, dim>(level0GridView, false);
-  EdgeAndFaceCommunication<typename GridType::LevelGridView, dim-1>::test(level0GridView);
+  testCommunication<LevelGV, 0>(level0GridView, false);
+  testCommunication<LevelGV, dim>(level0GridView, false);
+  EdgeAndFaceCommunication<LevelGV, dim-1>::test(level0GridView);
   if (dim == 3)
-    EdgeAndFaceCommunication<typename GridType::LevelGridView, 1>::test(level0GridView);
+    EdgeAndFaceCommunication<LevelGV, 1>::test(level0GridView);
 
   // Test element and node communication on leaf view
-  testCommunication<typename GridType::LeafGridView, 0>(leafGridView, true);
-  testCommunication<typename GridType::LeafGridView, dim>(leafGridView, true);
-  EdgeAndFaceCommunication<typename GridType::LeafGridView, dim-1>::test(leafGridView);
+  testCommunication<LeafGV, 0>(leafGridView, true);
+  testCommunication<LeafGV, dim>(leafGridView, true);
+  EdgeAndFaceCommunication<LeafGV, dim-1>::test(leafGridView);
   if (dim == 3)
-    EdgeAndFaceCommunication<typename GridType::LeafGridView, 1>::test(leafGridView);
+    EdgeAndFaceCommunication<LeafGV, 1>::test(leafGridView);
 
   ////////////////////////////////////////////////////
   //  Refine globally and test again
@@ -592,14 +562,17 @@ void testParallelUG(bool localRefinement)
     std::cout << "Local refinement\n";
 
     // mark all elements with x-coordinate < 0.5 for refinement
-    typename LeafGV::template Codim<0>::Iterator
-    it = grid->leafGridView().template begin<0>();
-    const typename LeafGV::template Codim<0>::Iterator
-    &endIt = grid->leafGridView().template end<0>();
-    for (; it != endIt; ++it) {
+    for (const auto& element : elements(grid->leafGridView())) {
       int nRefine = 1;
-      if (it->geometry().center()[0] < 0.5)
-        grid->mark(nRefine, *it);
+      bool refine;
+      if (refineUpperPart) {
+        refine = (element.geometry().center()[refinementDim] > 0.5);
+      }
+      else {
+        refine = (element.geometry().center()[refinementDim] < 0.5);
+      }
+      if (refine)
+        grid->mark(nRefine, element);
     }
 
     // adapt the grid
@@ -630,25 +603,24 @@ void testParallelUG(bool localRefinement)
 
   for (int i=0; i<=grid->maxLevel(); i++)
   {
-    testCommunication<typename GridType::LevelGridView, 0>(grid->levelGridView(i), false);
-    testCommunication<typename GridType::LevelGridView, dim>(grid->levelGridView(i), false);
-    EdgeAndFaceCommunication<typename GridType::LevelGridView, dim-1>::test(grid->levelGridView(i));
+    testCommunication<LevelGV, 0>(grid->levelGridView(i), false);
+    testCommunication<LevelGV, dim>(grid->levelGridView(i), false);
+    EdgeAndFaceCommunication<LevelGV, dim-1>::test(grid->levelGridView(i));
     if (dim == 3)
-      EdgeAndFaceCommunication<typename GridType::LevelGridView, 1>::test(grid->levelGridView(i));
+      EdgeAndFaceCommunication<LevelGV, 1>::test(grid->levelGridView(i));
   }
-  testCommunication<typename GridType::LeafGridView, 0>(grid->leafGridView(), true);
-  testCommunication<typename GridType::LeafGridView, dim>(grid->leafGridView(), true);
-  EdgeAndFaceCommunication<typename GridType::LeafGridView, dim-1>::test(grid->leafGridView());
+  testCommunication<LeafGV, 0>(grid->leafGridView(), true);
+  testCommunication<LeafGV, dim>(grid->leafGridView(), true);
+  EdgeAndFaceCommunication<LeafGV, dim-1>::test(grid->leafGridView());
   if (dim == 3)
-    EdgeAndFaceCommunication<typename GridType::LeafGridView, 1>::test(grid->leafGridView());
+    EdgeAndFaceCommunication<LeafGV, 1>::test(grid->leafGridView());
 
 }
 
 int main (int argc , char **argv) try
 {
   // initialize MPI, finalize is done automatically on exit
-  Dune::MPIHelper &mpiHelper =
-    Dune::MPIHelper::instance(argc, argv);
+  auto &mpiHelper = Dune::MPIHelper::instance(argc, argv);
 
   std::cout << "This is process "
             << mpiHelper.rank() + 1
@@ -658,17 +630,21 @@ int main (int argc , char **argv) try
             << getpid()
             << " .\n";
 
-  // test 2D grid with uniform refinement
-  testParallelUG<2>(false);
-
-  // test 3D grid with uniform refinement
-  testParallelUG<3>(false);
-
-  // test 2D grid with adaptive refinement
-  testParallelUG<2>(true);
-
-  // test 3D grid with adaptive refinement
-  testParallelUG<3>(true);
+  /*
+   * Test 2D and 3D grids,
+   * on structured cube and simplex grids,
+   * with global and local refinement.
+   */
+  for (const bool simplexGrid : {false, true}) {
+    for (const bool localRefinement : {false, true}) {
+      for (const bool refineUpperPart : {false, true}) {
+        for (const int refinementDim : {0,1})
+          testParallelUG<2>(simplexGrid, localRefinement, refinementDim, refineUpperPart);
+        for (const int refinementDim : {0,1,2})
+          testParallelUG<3>(simplexGrid, localRefinement, refinementDim, refineUpperPart);
+      }
+    }
+  }
 
   return 0;
 }
